package test;

import java.io.FileInputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.NamingException;

import org.jboss.ejb.client.naming.ejb.ejbURLContextFactory;
import org.junit.Test;

import api.ClientEjbService;
import thread.CallThread;

public class EjbTest {

	int[] threadSizes = { 150 };
	protected static int CALL_DURATION_IN_SECS = 5;

	private static final String JNDI_NAME = "ejb:ejbTest-client-ear-1.0.0-SNAPSHOT/ejbTest-client-ejb-1.0.0-SNAPSHOT/EjbClient!api.ClientEjbService";
	protected Context context;
	protected ClientEjbService service;
	DateFormat dfmt = new SimpleDateFormat("hh:mm:ss:S");

	public EjbTest() {
		try {
			context = getContext("src/test/resources/context0.properties");

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private Context getContext(String file) throws Exception {
		FileInputStream fileInputStream = new FileInputStream(file);
		Properties props = new Properties();
		props.load(fileInputStream);
		return (Context) new ejbURLContextFactory().getObjectInstance(null, null, null, props);
	}

	protected Context getLookupContext(int threadNumber) {
		return context;
	}

	public static void main(String[] args) throws Exception {
		new EjbTest().measureRunnableTimeAndThreadRise();
	}

	@Test
	public void test() throws Exception {
		measureRunnableTimeAndThreadRise();
	}

	public void measureRunnableTimeAndThreadRise() throws Exception {

		for (int i = 0; i < threadSizes.length; i++) {

			System.out.println(dfmt.format(new Date()) + " starting test");

			List<CallThread> threads = new ArrayList<CallThread>();

			// another problem in nightly build in WildFly:
			// starting all threads at once also blocks
			// so rampup with 1 thread only
			Thread rampup = getCallThread(000, 5);
			rampup.run();
			Thread.sleep(6000);
			if (rampup.isAlive()) {
				System.out.println(dfmt.format(new Date()) + " Rampup didn't work. Failing.");
				throw new RuntimeException("Rampup didn't work. Failing.");
			}

			// start calls
			for (int k = 0; k < threadSizes[i]; k++) {
				threads.add(getCallThread(k, CALL_DURATION_IN_SECS));
			}
			for (CallThread thread : threads) {
				thread.start();
			}

			waitForThreadsToFinish(threads);

			System.out.println(dfmt.format(new Date()) + " done");

		}

	}

	private void waitForThreadsToFinish(List<CallThread> threads) throws InterruptedException {
		long sleeptime = CALL_DURATION_IN_SECS * 1000 + 2000;
		System.out.println(dfmt.format(new Date()) + " sleeping " + sleeptime + "secs");
		Thread.sleep(sleeptime);

		boolean waitForFinish = true;
		int counter = 0;
		while (waitForFinish) {
			int threadsFinished = 0;
			for (CallThread thread : threads) {
				if (!thread.isAlive()) {
					threadsFinished++;
				} else {
					continue;
				}
			}
			if (threadsFinished == threads.size()) {
				System.out.println(dfmt.format(new Date()) + " all threads finished.");
				waitForFinish = false;
				continue;
			}

			// sleep another 2 secs
			System.out.println(dfmt.format(new Date()) + " not all threads finished. Waiting another 2 seconds...");

			// fail after 10 waits
			counter++;
			if (counter == 10) {
				System.out.println(dfmt.format(new Date()) + " Calls didn't return. Waited enough. Failing tests");
				throw new RuntimeException("Calls didn't return. Waited enough. Failing tests");
			}
			Thread.sleep(2000);
		}
	}

	private CallThread getCallThread(int threadNumber, long waitTime) throws NamingException {
		ClientEjbService service = (ClientEjbService) context.lookup(JNDI_NAME);
		return new CallThread(threadNumber, service, waitTime);
	}

}
